import requests

from metabase.exceptions import AuthenticationError


class Metabase:
    def __init__(self, host: str, user: str, password: str, token: str = None):
        self._host = host
        self.user = user
        self.password = password
        self._token = token

    @property
    def host(self):
        host = self._host

        if not host.startswith("http"):
            host = "https://" + self._host

        return host.rstrip("/")

    @host.setter
    def host(self, value):
        self._host = value

    @property
    def token(self):
        if self._token is None:
            response = requests.post(
                self.host + "/api/session",
                json={"username": self.user, "password": self.password},
            )

            if response.status_code != 200:
                raise AuthenticationError(response.content.decode())

            self._token = response.json()["id"]

        return self._token

    @token.setter
    def token(self, value):
        self._token = value

    @property
    def headers(self):
        return {"X-Metabase-Session": self.token}

    def get(self, endpoint: str, **kwargs):
        return requests.get(self.host + endpoint, headers=self.headers, **kwargs)

    def post(self, endpoint: str, **kwargs):
        return requests.post(self.host + endpoint, headers=self.headers, **kwargs)

    def put(self, endpoint: str, **kwargs):
        return requests.put(self.host + endpoint, headers=self.headers, **kwargs)

    def delete(self, endpoint: str, **kwargs):
        return requests.delete(self.host + endpoint, headers=self.headers, **kwargs)



from typing import List

from metabase.mbql.base import Mbql, Option


class Aggregation(Mbql):
    function: str

    def __init__(self, id: int, name: str = None, option: Option = None):
        self.name = name
        super(Aggregation, self).__init__(id=id, option=option)

    def compile(self) -> List:
        compiled = [self.function, super(Aggregation, self).compile()]

        if self.name is not None:
            compiled = self.compile_name(compiled, self.name)

        return compiled

    @staticmethod
    def compile_name(compiled, name: str) -> str:
        return (
            ["aggregation-options"]
            + [compiled]
            + [{"name": name, "display-name": name}]
        )


class Count(Aggregation):
    function = "count"

    def __init__(self, id: int = None, name: str = None, option: Option = None):
        self.id = id
        self.name = name

    def compile(self) -> List:
        compiled = [self.function]

        if self.name is not None:
            compiled = self.compile_name(compiled, self.name)

        return compiled


class Sum(Aggregation):
    function = "sum"


class Average(Aggregation):
    function = "avg"


class Distinct(Aggregation):
    function = "distinct"


class CumulativeSum(Aggregation):
    function = "cum-sum"


class CumulativeCount(Aggregation):
    function = "cum-count"


class StandardDeviation(Aggregation):
    function = "stddev"


class Min(Aggregation):
    function = "min"


class Max(Aggregation):
    function = "max"



from typing import Any, List

from metabase.mbql.base import Mbql, Option


class CaseOption(Option):
    CASE_SENSITIVE = {"case-sensitive": True}
    CASE_INSENSITIVE = {"case-sensitive": False}


class TimeGrainOption(Option):
    MINUTE = "minute"
    HOUR = "hour"
    DAY = "day"
    WEEK = "week"
    MONTH = "month"
    QUARTER = "quarter"
    YEAR = "year"


class Filter(Mbql):
    function: str

    def __init__(self, id: int, option: Option = None):
        self.id = id
        self.option = None
        self.filter_option = option

    def compile(self) -> List:
        compiled = [self.function, super(Filter, self).compile()]

        if self.filter_option is not None:
            compiled = compiled + [self.filter_option]

        return compiled


class ValueFilter(Filter):
    def __init__(self, id: int, value: Any, option: Option = None):
        self.id = id
        self.value = value
        self.option = None
        self.filter_option = option

    def compile(self) -> List:
        compiled = [self.function, super(Filter, self).compile(), self.value]

        if self.filter_option is not None:
            compiled = compiled + [self.filter_option]

        return compiled


class Equal(ValueFilter):
    function = "="


class NotEqual(ValueFilter):
    function = "!="


class Greater(ValueFilter):
    function = ">"


class Less(ValueFilter):
    function = "<"


class Between(Filter):
    function = "between"

    def __init__(
        self, id: int, lower_bound: float, upper_bound: float, option: Option = None
    ):
        self.id = id
        self.option = None
        self.filter_option = option
        self.lower_bound = lower_bound
        self.upper_bound = upper_bound

    def compile(self) -> List:
        return super(Between, self).compile() + [self.lower_bound, self.upper_bound]


class GreaterEqual(ValueFilter):
    function = ">="


class LessEqual(ValueFilter):
    function = "<="


class IsNull(Filter):
    function = "is-null"


class IsNotNull(Filter):
    function = "not-null"


class Contains(ValueFilter):
    function = "contains"


class StartsWith(ValueFilter):
    function = "starts-with"


class EndsWith(ValueFilter):
    function = "ends-with"


class TimeInterval(Filter):
    function = "time-interval"

    def __init__(
        self,
        id: int,
        value: Any,
        time_grain: TimeGrainOption,
        include_current: bool = True,
    ):
        self.id = id
        self.value = value
        self.option = None
        self.time_grain = time_grain
        self.include_current = include_current

    def compile(self) -> List:
        compiled = [
            self.function,
            super(Filter, self).compile(),
            self.value,
            self.time_grain,
        ]

        if self.include_current:
            compiled = compiled + [{"include-current": True}]

        return compiled



from __future__ import annotations

from typing import List

from metabase import Metabase
from metabase.missing import MISSING
from metabase.resource import CreateResource, GetResource, ListResource, UpdateResource


class Segment(ListResource, CreateResource, GetResource, UpdateResource):
    ENDPOINT = "/api/segment"

    id: int
    table_id: int
    creator_id: int

    name: str
    description: str
    definition: dict
    points_of_interest: str
    caveats: str

    show_in_getting_started: bool
    archived: bool

    updated_at: str
    created_at: str

    @classmethod
    def list(cls, using: Metabase) -> List[Segment]:
        return super(Segment, cls).list(using=using)

    @classmethod
    def get(cls, id: int, using: Metabase) -> Segment:
        return super(Segment, cls).get(id, using=using)

    @classmethod
    def create(
        cls,
        using: Metabase,
        name: str,
        table_id: int,
        definition: dict,
        description: str = "Created by metabase-python.",
        **kwargs,
    ) -> Segment:
        return super(Segment, cls).create(
            using=using,
            name=name,
            table_id=table_id,
            definition=definition,
            description=description,
            **kwargs,
        )

    def update(
        self,
        revision_message: str = "Updated by metabase-python.",
        name: str = MISSING,
        description: str = MISSING,
        definition: dict = MISSING,
        show_in_getting_started: str = MISSING,
        points_of_interest: str = MISSING,
        caveats: str = MISSING,
        archived: bool = MISSING,
        **kwargs,
    ) -> None:
        return super(Segment, self).update(
            revision_message=revision_message,
            name=name,
            description=description,
            definition=definition,
            points_of_interest=points_of_interest,
            caveats=caveats,
            archived=archived,
            show_in_getting_started=show_in_getting_started,
            **kwargs,
        )

    def archive(self):
        """Archive a Segment."""
        return self.update(
            archived=True, revision_message="Archived by metabase-python."
        )



from __future__ import annotations

from enum import Enum
from typing import Any, Dict, List, Optional

from metabase import Metabase
from metabase.missing import MISSING
from metabase.resource import GetResource, UpdateResource


class Field(GetResource, UpdateResource):
    ENDPOINT = "/api/field"

    id: int
    table_id: int
    fk_target_id: Optional[int]
    parent_id: Optional[int]

    name: str
    display_name: str
    description: str

    database_type: str
    semantic_type: Field.SemanticType
    effective_type: str
    base_type: str

    dimensions: List[str]
    dimension_options: List[str]
    default_dimension_option: Optional[int]

    database_position: int
    custom_position: int
    visibility_type: str
    points_of_interest: str
    has_field_values: Field.FieldValue

    active: bool
    preview_display: bool

    target: Optional[Field]

    settings: dict
    caveats: str
    coercion_strategy: str

    updated_at: str
    created_at: str

    class FieldValue(str, Enum):
        none = "none"
        auto_list = "auto-list"
        list = "list"
        search = "search"

    class SemanticType(str, Enum):
        primary_key = "Type/PK"
        foreign_key = "Type/FK"

        avatar_url = "type/AvatarURL"
        birthdate = "type/Birthdate"
        cancelation_date = "type/CancelationDate"
        cancelation_time = "type/CancelationTime"
        cancelation_timestamp = "type/CancelationTimestamp"
        category = "type/Category"
        city = "type/City"
        comment = "type/Comment"
        company = "type/Company"
        cost = "type/Cost"
        country = "type/Country"
        creation_date = "type/CreationDate"
        creation_time = "type/CreationTime"
        creation_timestamp = "type/CreationTimestamp"
        currency = "type/Currency"
        deletion_date = "type/DeletionDate"
        deletion_time = "type/DeletionTime"
        deletion_timestamp = "type/DeletionTimestamp"
        description = "type/Description"
        discount = "type/Discount"
        email = "type/Email"
        enum = "type/Enum"
        gross_margin = "type/GrossMargin"
        image_url = "type/ImageURL"
        income = "type/Income"
        join_date = "type/JoinDate"
        join_time = "type/JoinTime"
        join_timestamp = "type/JoinTimestamp"
        latitude = "type/Latitude"
        longitude = "type/Longitude"
        name = "type/Name"
        number = "type/Number"
        owner = "type/Owner"
        price = "type/Price"
        product = "type/Product"
        quantity = "type/Quantity"
        score = "type/Score"
        serialized_json = "type/SerializedJSON"
        share = "type/Share"
        source = "type/Source"
        state = "type/State"
        subscription = "type/Subscription"
        title = "type/Title"
        url = "type/URL"
        user = "type/User"
        zip_code = "type/ZipCode"

    class VisibilityType(str, Enum):
        details_only = "details-only"
        hidden = "hidden"
        normal = "normal"
        retired = "retired"
        sensitive = "sensitive"

    @classmethod
    def get(cls, id: int, using: Metabase) -> Field:
        """Get Field with ID."""
        return super(Field, cls).get(id, using=using)

    def update(
        self,
        display_name: str = MISSING,
        description: str = MISSING,
        semantic_type: Field.SemanticType = MISSING,
        visibility_type: Field.VisibilityType = MISSING,
        fk_target_field_id: int = MISSING,
        has_field_values: Field.FieldValue = MISSING,
        points_of_interest: str = MISSING,
        settings: str = MISSING,
        caveats: str = MISSING,
        coercion_strategy: str = MISSING,
        **kwargs,
    ) -> None:
        """Update Field with ID."""
        return super(Field, self).update(
            display_name=display_name,
            description=description,
            semantic_type=semantic_type,
            visibility_type=visibility_type,
            fk_target_field_id=fk_target_field_id,
            has_field_values=has_field_values,
            points_of_interest=points_of_interest,
            settings=settings,
            caveats=caveats,
            coercion_strategy=coercion_strategy,
        )

    def related(self) -> Dict[str, Any]:
        """Return related entities."""
        return self._using.get(
            self.ENDPOINT + f"/{getattr(self, self.PRIMARY_KEY)}" + "/related"
        ).json()

    def discard_values(self):
        """
        Discard the FieldValues belonging to this Field. Only applies to fields
        that have FieldValues. If this Field’s Database is set up to automatically
        sync FieldValues, they will be recreated during the next cycle.

        You must be a superuser to do this.
        """
        return self._using.post(
            self.ENDPOINT + f"/{getattr(self, self.PRIMARY_KEY)}" + "/discard_values"
        )

    def rescan_values(self):
        """
        Manually trigger an update for the FieldValues for this Field. Only applies
        to Fields that are eligible for FieldValues.

        You must be a superuser to do this.
        """
        return self._using.post(
            self.ENDPOINT + f"/{getattr(self, self.PRIMARY_KEY)}" + "/rescan_values"
        )



from __future__ import annotations

from typing import Any, List

import pandas as pd

from metabase import Metabase
from metabase.resource import CreateResource, Resource


class Data(Resource):
    ENDPOINT = None
    PRIMARY_KEY = None

    rows: List[List[Any]]
    cols: dict
    native_form: dict

    def to_pandas(self):
        """Returns the query results as a Pandas DataFrame."""
        columns = [col["display_name"] for col in self.cols]
        return pd.DataFrame(data=self.rows, columns=columns)


class Dataset(CreateResource):
    ENDPOINT = "/api/dataset"
    PRIMARY_KEY = None

    context: str
    status: str
    database_id: int
    data: Data
    row_count: int

    started_at: str
    running_time: int
    json_query: str
    average_execution_time: int = None

    @classmethod
    def create(
        cls, using: Metabase, database: int, type: str, query: dict, **kwargs
    ) -> Dataset:
        """Execute a query and retrieve the results in the usual format."""
        dataset = super(Dataset, cls).create(
            using=using, database=database, type=type, query=query
        )
        dataset.data = Data(_using=using, **dataset.data)
        return dataset

    def to_pandas(self) -> pd.DataFrame:
        """Returns the query results as a Pandas DataFrame."""
        return self.data.to_pandas()



from __future__ import annotations

from enum import Enum
from typing import Any, Dict, List

from metabase import Metabase
from metabase.missing import MISSING
from metabase.resource import GetResource, ListResource, Resource, UpdateResource
from metabase.resources.field import Field
from metabase.resources.metric import Metric
from metabase.resources.segment import Segment


class Dimension(Resource):
    ENDPOINT = None

    id: str
    name: str
    mbql: List[Any]
    type: str


class Table(ListResource, GetResource, UpdateResource):
    ENDPOINT = "/api/table"

    id: int
    name: str
    schema: str
    db_id: int
    db: dict

    display_name: str
    description: str

    entity_name: str
    entity_type: str

    pk_field: int
    visibility_type: Table.VisibilityType
    field_order: Table.FieldOrder
    points_of_interest: str

    active: bool
    show_in_getting_started: bool

    created_at: str
    updated_at: str

    _metrics = None
    _fields = None

    class VisibilityType(str, Enum):
        cruft = "cruft"
        hidden = "hidden"
        technical = "technical"

    class FieldOrder(str, Enum):
        alphabetical = "alphabetical"
        custom = "custom"
        database = "database"
        smart = "smart"

    @classmethod
    def list(cls, using: Metabase) -> List[Table]:
        """Get all Tables."""
        return super(Table, cls).list(using=using)

    @classmethod
    def get(cls, id: int, using: Metabase) -> Table:
        """Get Table with ID."""
        return super(Table, cls).get(id, using=using)

    def update(
        self,
        display_name: str = MISSING,
        description: str = MISSING,
        field_order: Table.FieldOrder = MISSING,
        visibility_type: Table.VisibilityType = MISSING,
        entity_type: str = MISSING,
        points_of_interest: str = MISSING,
        caveats: str = MISSING,
        show_in_getting_started: bool = MISSING,
        **kwargs,
    ) -> None:
        """Update Table with ID."""
        return super(Table, self).update(
            display_name=display_name,
            description=description,
            field_order=field_order,
            visibility_type=visibility_type,
            entity_type=entity_type,
            points_of_interest=points_of_interest,
            caveats=caveats,
            show_in_getting_started=show_in_getting_started,
        )

    def fks(self) -> List[dict]:
        """Get all foreign keys whose destination is a Field that belongs to this Table."""
        return self._using.get(
            self.ENDPOINT + f"/{getattr(self, self.PRIMARY_KEY)}" + "/fks"
        ).json()

    def query_metadata(self) -> Dict[str, Any]:
        """
        Get metadata about a Table useful for running queries. Returns DB, fields,
        field FKs, and field values.

        Passing include_hidden_fields=true will include any hidden Fields in the response.
        Defaults to false Passing include_sensitive_fields=true will include any sensitive
        Fields in the response. Defaults to false.

        These options are provided for use in the Admin Edit Metadata page.
        """
        return self._using.get(
            self.ENDPOINT + f"/{getattr(self, self.PRIMARY_KEY)}" + "/query_metadata"
        ).json()

    def related(self) -> Dict[str, Any]:
        """Return related entities."""
        return self._using.get(
            self.ENDPOINT + f"/{getattr(self, self.PRIMARY_KEY)}" + "/related"
        ).json()

    def discard_values(self):
        """
        Discard the FieldValues belonging to the Fields in this Table. Only applies to
        fields that have FieldValues. If this Table’s Database is set up to automatically
        sync FieldValues, they will be recreated during the next cycle.

        You must be a superuser to do this.
        """
        self._using.post(
            self.ENDPOINT + f"/{getattr(self, self.PRIMARY_KEY)}" + "/discard_values"
        )

    def rescan_values(self):
        """
        Manually trigger an update for the FieldValues for the Fields belonging to this Table.
        Only applies to Fields that are eligible for FieldValues.

        You must be a superuser to do this.
        """
        self._using.post(
            self.ENDPOINT + f"/{getattr(self, self.PRIMARY_KEY)}" + "/rescan_values"
        )

    def fields(self) -> List[Field]:
        """Get all Fields associated with this Table.."""
        return [
            Field(_using=self._using, **field)
            for field in self.query_metadata().get("fields")
        ]

    def dimensions(self) -> List[Dimension]:
        """Get all Dimensions associated with this Table."""
        return [
            Dimension(id=id, _using=self._using, **dimension)
            for id, dimension in self.query_metadata()
            .get("dimension_options", {})
            .items()
        ]

    def metrics(self) -> List[Metric]:
        """Get all Metrics associated with this Table."""
        return [
            Metric(_using=self._using, **metric)
            for metric in self.related().get("metrics")
        ]

    def segments(self) -> List[Segment]:
        """Get all Segments associated with this Table."""
        return [
            Segment(_using=self._using, **segment)
            for segment in self.related().get("segments")
        ]



from __future__ import annotations

from typing import List

from requests import HTTPError

from metabase import Metabase
from metabase.resource import CreateResource, DeleteResource, ListResource


class PermissionMembership(ListResource, CreateResource, DeleteResource):
    ENDPOINT = "/api/permissions/membership"
    PRIMARY_KEY = "membership_id"

    membership_id: int
    group_id: int
    user_id: int

    # TODO: allow for bulk updates through /api/permissions/membership/graph

    @classmethod
    def list(cls, using: Metabase) -> List[PermissionMembership]:
        """
        Fetch a map describing the group memberships of various users. This map’s format is:

        {<user-id> [{:membership_id <id>
                     :group_id      <id>}]}.
        You must be a superuser to do this.
        """
        response = using.get(cls.ENDPOINT)
        all_memberships = [
            item for sublist in response.json().values() for item in sublist
        ]
        records = [cls(_using=using, **record) for record in all_memberships]
        return records

    @classmethod
    def create(
        cls, using: Metabase, group_id: int, user_id: int, **kwargs
    ) -> PermissionMembership:
        """
        Add a User to a PermissionsGroup. Returns updated list of members belonging to the group.

        You must be a superuser to do this.
        """
        response = using.post(
            cls.ENDPOINT, json={"group_id": group_id, "user_id": user_id}
        )

        if response.status_code != 200:
            raise HTTPError(response.content.decode())

        # metabase returns a list of all memberships for the given group_id
        membership = next(filter(lambda x: x["user_id"] == user_id, response.json()))

        return cls(_using=using, **membership)



from __future__ import annotations

from datetime import datetime
from typing import Any, Dict, List

from metabase import Metabase
from metabase.missing import MISSING
from metabase.resource import (
    CreateResource,
    DeleteResource,
    GetResource,
    ListResource,
    UpdateResource,
)


class User(ListResource, CreateResource, GetResource, UpdateResource, DeleteResource):
    ENDPOINT = "/api/user"

    id: int
    email: str
    first_name: str
    last_name: str
    common_name: str
    locale: str

    is_superuser: bool
    is_active: bool
    is_qbnewb: bool
    ldap_auth: bool
    google_auth: bool

    login_attributes: Dict[str, Any]
    group_ids: List[int]

    last_login: datetime
    date_joined: datetime
    updated_at: datetime

    @classmethod
    def list(
        cls,
        using: Metabase,
        status: str = None,
        query: str = None,
        group_id: int = None,
        include_deactivated: bool = None,
        limit: int = None,
        offset: int = None,
    ) -> List[User]:
        """
        Fetch a list of Users. By default returns every active user but only active users.

        If status is deactivated, include deactivated users only. If status is all, include all users (active and
        inactive). Also supports include_deactivated, which if true, is equivalent to status=all. status and
        included_deactivated requires superuser permissions.

        For users with segmented permissions, return only themselves.

        Takes limit, offset for pagination. Takes query for filtering on first name, last name, email. Also takes
        group_id, which filters on group id.
        """
        response = using.get(
            cls.ENDPOINT,
            params={
                "status": status,
                "query": query,
                "group_id": group_id,
                "include_deactivated": include_deactivated,
                "limit": limit,
                "offset": offset,
            },
        )
        records = [
            cls(_using=using, **user) for user in response.json().get("data", [])
        ]
        return records

    @classmethod
    def create(
        cls,
        using: Metabase,
        first_name: str,
        last_name: str,
        email: str,
        password: str,
        group_ids: List[int] = None,
        login_attributes: Dict[str, Any] = None,
        **kwargs,
    ) -> User:
        """
        Create a new User, return a 400 if the email address is already taken.

        You must be a superuser to do this.
        """
        return super(User, cls).create(
            using=using,
            first_name=first_name,
            last_name=last_name,
            email=email,
            password=password,
            group_ids=group_ids,
            login_attributes=login_attributes,
            **kwargs,
        )

    def update(
        self,
        first_name: str = MISSING,
        last_name: str = MISSING,
        email: str = MISSING,
        group_ids: List[int] = MISSING,
        is_superuser: bool = MISSING,
        locale: str = MISSING,
        **kwargs,
    ) -> None:
        """Update an existing, active User."""
        return super(User, self).update(
            first_name=first_name,
            last_name=last_name,
            email=email,
            group_ids=group_ids,
            locale=locale,
            **kwargs,
        )

    def delete(self) -> None:
        """
        Disable a User. This does not remove the User from the DB, but instead disables their account.

        You must be a superuser to do this.
        """
        return super(User, self).delete()

    def password(self, password: str, old_password: str):
        """Update a user’s password."""
        return self._using.put(
            self.ENDPOINT + f"/{getattr(self, self.PRIMARY_KEY)}" + "/password",
            json={"password": password, "old_password": old_password},
        )

    def send_invite(self):
        """
        Resend the user invite email for a given user.

        You must be a superuser to do this.
        """
        return self._using.put(
            self.ENDPOINT + f"/{getattr(self, self.PRIMARY_KEY)}" + "/send_invite"
        )

    def reactivate(self):
        """
        Reactivate user at :id.

        You must be a superuser to do this.
        """
        return self._using.put(
            self.ENDPOINT + f"/{getattr(self, self.PRIMARY_KEY)}" + "/reactivate"
        )



from metabase.mbql.base import Mbql, Option


class TemporalOption(Option):
    MINUTE = {"temporal-unit": "minute"}
    HOUR = {"temporal-unit": "hour"}
    DAY = {"temporal-unit": "day"}
    WEEK = {"temporal-unit": "week"}
    MONTH = {"temporal-unit": "month"}
    QUARTER = {"temporal-unit": "quarter"}
    YEAR = {"temporal-unit": "year"}
    MINUTE_OF_HOUR = {"temporal-unit": "minute-of-hour"}
    HOUR_OF_DAY = {"temporal-unit": "hour-of-day"}
    DAY_OF_WEEK = {"temporal-unit": "day-of-week"}
    DAY_OF_MONTH = {"temporal-unit": "day-of-month"}
    DAY_OF_YEAR = {"temporal-unit": "day-of-year"}
    WEEK_OF_YEAR = {"temporal-unit": "week-of-year"}
    MONTH_OF_YEAR = {"temporal-unit": "month-of-year"}
    QUARTER_OF_YEAR = {"temporal-unit": "quarter-of-year"}


class BinOption(Option):
    AUTO = {"binning": {"strategy": "default"}}
    BINS_10 = {"binning": {"strategy": "num-bins", "num-bins": 10}}
    BINS_50 = {"binning": {"strategy": "num-bins", "num-bins": 50}}
    BINS_100 = {"binning": {"strategy": "num-bins", "num-bins": 100}}
    NONE = None


class GroupBy(Mbql):
    pass



from typing import List


class Option:
    pass


class Mbql:
    def __init__(self, id: int, option: Option = None):
        self.id = id
        self.option = option

    def compile(self) -> List:
        return ["field", self.id, self.option]

    def __repr__(self):
        return str(self.compile())



from __future__ import annotations

from datetime import datetime
from typing import List

from metabase.metabase import Metabase
from metabase.missing import MISSING
from metabase.resource import CreateResource, GetResource, ListResource, UpdateResource


class Metric(ListResource, CreateResource, GetResource, UpdateResource):
    ENDPOINT = "/api/metric"

    id: int
    database_id: int
    table_id: int
    creator_id: dict

    name: str
    description: str
    definition: dict
    how_is_this_calculated: str
    points_of_interest: str
    caveats: str

    archived: bool
    show_in_getting_started: bool

    created_at: datetime
    updated_at: datetime

    creator: dict

    @classmethod
    def list(cls, using: Metabase) -> List[Metric]:
        return super(Metric, cls).list(using=using)

    @classmethod
    def get(cls, id: int, using: Metabase) -> Metric:
        return super(Metric, cls).get(id, using=using)

    @classmethod
    def create(
        cls,
        using: Metabase,
        name: str,
        table_id: int,
        definition: dict,
        description: str = "Created by metabase-python.",
        **kwargs,
    ) -> Metric:
        return super(Metric, cls).create(
            using=using,
            name=name,
            table_id=table_id,
            definition=definition,
            description=description,
            **kwargs,
        )

    def update(
        self,
        revision_message: str = "Updated by metabase-python.",
        name: str = MISSING,
        description: str = MISSING,
        definition: dict = MISSING,
        how_is_this_calculated: str = MISSING,
        points_of_interest: str = MISSING,
        caveats: str = MISSING,
        archived: bool = MISSING,
        show_in_getting_started: bool = MISSING,
        **kwargs,
    ) -> None:
        return super(Metric, self).update(
            revision_message=revision_message,
            name=name,
            description=description,
            definition=definition,
            how_is_this_calculated=how_is_this_calculated,
            points_of_interest=points_of_interest,
            caveats=caveats,
            archived=archived,
            show_in_getting_started=show_in_getting_started,
            **kwargs,
        )

    def archive(self):
        """Archive a Metric."""
        return self.update(
            archived=True, revision_message="Archived by metabase-python."
        )



from __future__ import annotations

from typing import List

from metabase import Metabase
from metabase.missing import MISSING
from metabase.resource import CreateResource, GetResource, ListResource, UpdateResource


class Card(ListResource, CreateResource, GetResource, UpdateResource):
    ENDPOINT = "/api/card"

    id: int
    table_id: int
    database_id: int
    collection_id: int
    creator_id: int
    made_public_by_id: int
    public_uuid: str

    name: str
    description: str

    collection: dict  # TODO: Collection
    collection_position: int

    query_type: str
    dataset_query: dict  # TODO: DatasetQuery
    display: str
    visualization_settings: dict  # TODO: VisualizationSettings
    result_metadata: List[dict]

    embedding_params: dict
    cache_ttl: str
    creator: "User"

    favorite: bool
    archived: bool
    enable_embedding: bool

    updated_at: str
    created_at: str

    @classmethod
    def list(cls, using: Metabase) -> List[Card]:
        """
        Get all the Cards. Option filter param f can be used to change the set
        of Cards that are returned; default is all, but other options include
        mine, fav, database, table, recent, popular, and archived.

        See corresponding implementation functions above for the specific behavior
        of each filter option. :card_index:.
        """
        # TODO: add support for endpoint parameters: f, model_id.
        return super(Card, cls).list(using)

    @classmethod
    def get(cls, id: int, using: Metabase) -> Card:
        """
        Get Card with ID.
        """
        return super(Card, cls).get(id, using)

    @classmethod
    def create(
        cls,
        using: Metabase,
        name: str,
        dataset_query: dict,  # TODO: DatasetQuery
        visualization_settings: dict,  # TODO: VisualizationSettings
        display: str,
        description: str = None,
        collection_id: str = None,
        collection_position: int = None,
        result_metadata: List[dict] = None,
        metadata_checksum: str = None,
        cache_ttl: int = None,
        **kwargs,
    ) -> Card:
        """
        Create a new Card.
        """
        return super(Card, cls).create(
            using=using,
            name=name,
            dataset_query=dataset_query,
            visualization_settings=visualization_settings,
            display=display,
            description=description,
            collection_id=collection_id,
            collection_position=collection_position,
            result_metadata=result_metadata,
            metadata_checksum=metadata_checksum,
            cache_ttl=cache_ttl,
            **kwargs,
        )

    def update(
        self,
        name: str = MISSING,
        dataset_query: dict = MISSING,  # TODO: DatasetQuery
        visualization_settings: dict = MISSING,  # TODO: VisualizationSettings
        display: str = MISSING,
        description: str = MISSING,
        collection_id: str = MISSING,
        collection_position: int = MISSING,
        result_metadata: List[dict] = MISSING,
        metadata_checksum: str = MISSING,
        archived: bool = MISSING,
        enable_embedding: bool = MISSING,
        embedding_params: dict = MISSING,
        cache_ttl: int = None,
        **kwargs,
    ) -> None:
        """
        Update a Card.
        """
        return super(Card, self).update(
            name=name,
            dataset_query=dataset_query,
            visualization_settings=visualization_settings,
            display=display,
            description=description,
            collection_id=collection_id,
            collection_position=collection_position,
            result_metadata=result_metadata,
            metadata_checksum=metadata_checksum,
            archived=archived,
            enable_embedding=enable_embedding,
            embedding_params=embedding_params,
            cache_ttl=cache_ttl,
        )

    def archive(self):
        """Archive a Card."""
        return self.update(
            archived=True, revision_message="Archived by metabase-python."
        )



from dataclasses import dataclass, field
from typing import List, Union

from metabase.mbql.aggregations import Aggregation
from metabase.mbql.filter import Filter
from metabase.mbql.groupby import GroupBy
from metabase.resources.metric import Metric


@dataclass
class Query:
    table_id: int
    aggregations: List[Union[Aggregation, Metric]]
    group_by: List[GroupBy] = field(default_factory=list)
    filters: List[Filter] = field(default_factory=list)

    def compile(self):
        return {
            "source-table": self.table_id,
            "aggregation": self._aggregations,
            "breakout": self._group_by,
            "filter": self._filters,
        }

    @property
    def _aggregations(self):
        aggregations = []
        for aggregation in self.aggregations:
            if isinstance(aggregation, Metric):
                aggregations.append(["metric", aggregation.id])
            else:
                aggregations.append(aggregation.compile())

        return aggregations

    @property
    def _group_by(self):
        return [group.compile() for group in self.group_by]

    @property
    def _filters(self):
        if len(self.filters) == 0:
            return self.filters

        if len(self.filters) == 1:
            return self.filters[0].compile()

        return ["and"] + [filt.compile() for filt in self.filters]



from __future__ import annotations

from requests import HTTPError

from metabase.exceptions import NotFoundError
from metabase.metabase import Metabase
from metabase.missing import MISSING


class Resource:
    ENDPOINT: str
    PRIMARY_KEY: str = "id"

    def __init__(self, _using: Metabase, **kwargs):
        self._attributes = []
        self._using = _using

        for k, v in kwargs.items():
            self._attributes.append(k)
            setattr(self, k, v)

    def __repr__(self):
        # move primary key to beginning of the list
        attributes = self._attributes.copy()
        if self.PRIMARY_KEY is not None:
            attributes.insert(0, attributes.pop(attributes.index(self.PRIMARY_KEY)))

        return (
            self.__class__.__qualname__
            + "("
            + ", ".join([f"{attr}={getattr(self, attr)}" for attr in attributes])
            + ")"
        )


class ListResource(Resource):
    @classmethod
    def list(cls, using: Metabase):
        """List all instances."""
        response = using.get(cls.ENDPOINT)
        records = [cls(_using=using, **record) for record in response.json()]
        return records


class GetResource(Resource):
    @classmethod
    def get(cls, id: int, using: Metabase):
        """Get a single instance by ID."""
        response = using.get(cls.ENDPOINT + f"/{id}")

        if response.status_code == 404 or response.status_code == 204:
            raise NotFoundError(f"{cls.__name__}(id={id}) was not found.")

        return cls(_using=using, **response.json())


class CreateResource(Resource):
    @classmethod
    def create(cls, using: Metabase, **kwargs):
        """Create an instance and save it."""
        response = using.post(cls.ENDPOINT, json=kwargs)

        if response.status_code not in (200, 202):
            raise HTTPError(response.content.decode())

        return cls(_using=using, **response.json())


class UpdateResource(Resource):
    def update(self, **kwargs) -> None:
        """
        Update an instance by providing function arguments.
        Providing any argument with metabase.MISSING will result in this argument being
        ignored from the request.
        """
        params = {k: v for k, v in kwargs.items() if v != MISSING}
        response = self._using.put(
            self.ENDPOINT + f"/{getattr(self, self.PRIMARY_KEY)}", json=params
        )

        if response.status_code not in (200, 202):
            raise HTTPError(response.json())

        for k, v in kwargs.items():
            setattr(self, k, v)


class DeleteResource(Resource):
    def delete(self) -> None:
        """Delete an instance."""
        response = self._using.delete(
            self.ENDPOINT + f"/{getattr(self, self.PRIMARY_KEY)}"
        )

        if response.status_code not in (200, 204):
            raise HTTPError(response.content.decode())



from metabase.mbql.aggregations import (
    Average,
    Count,
    CumulativeCount,
    CumulativeSum,
    Distinct,
    Max,
    Min,
    StandardDeviation,
    Sum,
)
from metabase.mbql.filter import (
    Between,
    CaseOption,
    EndsWith,
    Equal,
    Greater,
    GreaterEqual,
    IsNotNull,
    IsNull,
    Less,
    LessEqual,
    NotEqual,
    StartsWith,
)
from metabase.mbql.groupby import BinOption, GroupBy, TemporalOption
from metabase.mbql.query import Query
from metabase.metabase import Metabase
from metabase.resources.card import Card
from metabase.resources.database import Database
from metabase.resources.dataset import Dataset
from metabase.resources.field import Field
from metabase.resources.metric import Metric
from metabase.resources.permission_group import PermissionGroup
from metabase.resources.permission_membership import PermissionMembership
from metabase.resources.segment import Segment
from metabase.resources.table import Table
from metabase.resources.user import User



from __future__ import annotations

from typing import List

from metabase import Metabase
from metabase.resource import (
    CreateResource,
    DeleteResource,
    GetResource,
    ListResource,
    UpdateResource,
)


class PermissionGroup(
    ListResource, CreateResource, GetResource, UpdateResource, DeleteResource
):
    ENDPOINT = "/api/permissions/group"

    id: int
    name: str
    member_count: int

    @classmethod
    def list(cls, using: Metabase) -> List[PermissionGroup]:
        """
        Fetch all PermissionsGroups, including a count of the number of :members in that group.

        You must be a superuser to do this.
        """
        return super(PermissionGroup, cls).list(using=using)

    @classmethod
    def get(cls, id: int, using: Metabase) -> PermissionGroup:
        """
        Fetch the details for a certain permissions group.

        You must be a superuser to do this.
        """
        return super(PermissionGroup, cls).get(id, using=using)

    @classmethod
    def create(cls, using: Metabase, name: str, **kwargs) -> PermissionGroup:
        """
        Create a new PermissionsGroup.

        You must be a superuser to do this.
        """
        return super(PermissionGroup, cls).create(using=using, name=name, **kwargs)

    def update(self, name: str, **kwargs) -> None:
        """
        Update the name of a PermissionsGroup.

        You must be a superuser to do this.
        """
        return super(PermissionGroup, self).update(name=name, **kwargs)

    def delete(self) -> None:
        """
        Delete a specific PermissionsGroup.

        You must be a superuser to do this.
        """
        return super(PermissionGroup, self).delete()



from __future__ import annotations

from typing import Any, Dict, List

from metabase import Metabase
from metabase.missing import MISSING
from metabase.resource import (
    CreateResource,
    DeleteResource,
    GetResource,
    ListResource,
    UpdateResource,
)
from metabase.resources.field import Field
from metabase.resources.table import Table


class Database(
    ListResource, CreateResource, GetResource, UpdateResource, DeleteResource
):
    ENDPOINT = "/api/database"

    id: int
    name: str
    description: str
    engine: str

    features: List[Any]
    details: Dict[str, str]
    options: str
    native_permissions: str

    timezone: str
    metadata_sync_schedule: str
    cache_field_values_schedule: str
    cache_ttl: int

    caveats: str
    refingerprint: str
    points_of_interest: str

    auto_run_queries: bool
    is_full_sync: bool
    is_on_demand: bool
    is_sample: bool

    updated_at: str
    created_at: str

    @classmethod
    def list(cls, using: Metabase) -> List[Database]:
        response = using.get(cls.ENDPOINT)
        records = [cls(_using=using, **db) for db in response.json().get("data", [])]
        return records

    @classmethod
    def get(cls, id: int, using: Metabase) -> Database:
        return super(Database, cls).get(id, using=using)

    @classmethod
    def create(
        cls,
        using: Metabase,
        name: str,
        engine: str,
        details: dict,
        is_full_sync: bool = None,
        is_on_demand: bool = None,
        schedules: dict = None,
        auto_run_queries: bool = None,
        cache_ttl: int = None,
        **kwargs,
    ) -> Database:
        """
        Add a new Database.

        You must be a superuser to do this.
        """
        return super(Database, cls).create(
            using=using,
            name=name,
            engine=engine,
            details=details,
            is_full_sync=is_full_sync,
            is_on_demand=is_on_demand,
            schedules=schedules,
            auto_run_queries=auto_run_queries,
            cache_ttl=cache_ttl,
            **kwargs,
        )

    def update(
        self,
        name: str = MISSING,
        description: str = MISSING,
        engine: str = MISSING,
        schedules: dict = MISSING,
        refingerprint: bool = MISSING,
        points_of_interest: str = MISSING,
        auto_run_queries: bool = MISSING,
        caveats: str = MISSING,
        is_full_sync: bool = MISSING,
        cache_ttl: int = MISSING,
        details: dict = MISSING,
        is_on_demand: bool = MISSING,
        **kwargs,
    ) -> None:
        """
        Update a Database.

        You must be a superuser to do this.
        """
        return super(Database, self).update(
            name=name,
            description=description,
            engine=engine,
            schedules=schedules,
            refingerprint=refingerprint,
            points_of_interest=points_of_interest,
            auto_run_queries=auto_run_queries,
            caveats=caveats,
            is_full_sync=is_full_sync,
            cache_ttl=cache_ttl,
            details=details,
            is_on_demand=is_on_demand,
        )

    def delete(self) -> None:
        """Delete a Database."""
        return super(Database, self).delete()

    def fields(self) -> List[Field]:
        """Get a list of all Fields in Database."""
        fields = self._using.get(
            self.ENDPOINT + f"/{getattr(self, self.PRIMARY_KEY)}" + "/fields"
        ).json()
        return [Field(_using=self._using, **payload) for payload in fields]

    def idfields(self) -> List[Field]:
        """Get a list of all primary key Fields for Database."""
        fields = self._using.get(
            self.ENDPOINT + f"/{getattr(self, self.PRIMARY_KEY)}" + "/idfields"
        ).json()
        return [Field(_using=self._using, **payload) for payload in fields]

    def schemas(self) -> List[str]:
        """Returns a list of all the schemas found for the database id."""
        return self._using.get(
            self.ENDPOINT + f"/{getattr(self, self.PRIMARY_KEY)}" + "/schemas"
        ).json()

    def tables(self, schema: str) -> List[Table]:
        """Returns a list of Tables for the given Database id and schema."""
        tables = self._using.get(
            self.ENDPOINT
            + f"/{getattr(self, self.PRIMARY_KEY)}"
            + "/schema"
            + f"/{schema}"
        ).json()
        return [Table(_using=self._using, **payload) for payload in tables]

    def discard_values(self):
        """
        Discards all saved field values for this Database.

        You must be a superuser to do this.
        """
        return self._using.post(
            self.ENDPOINT + f"/{getattr(self, self.PRIMARY_KEY)}" + "/discard_values"
        )

    def rescan_values(self):
        """
        Trigger a manual scan of the field values for this Database.

        You must be a superuser to do this.
        """
        return self._using.post(
            self.ENDPOINT + f"/{getattr(self, self.PRIMARY_KEY)}" + "/rescan_values"
        )

    def sync(self):
        """Update the metadata for this Database. This happens asynchronously."""
        return self._using.post(
            self.ENDPOINT + f"/{getattr(self, self.PRIMARY_KEY)}" + "/sync"
        )

    def sync_schema(self):
        """
        Trigger a manual update of the schema metadata for this Database.

        You must be a superuser to do this.
        """
        return self._using.post(
            self.ENDPOINT + f"/{getattr(self, self.PRIMARY_KEY)}" + "/sync_schema"
        )



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['metabase', 'metabase.mbql', 'metabase.resources']

package_data = \
{'': ['*']}

install_requires = \
['pandas>=1.0.0,<2.0.0', 'requests']

setup_kwargs = {
    'name': 'metabase-python',
    'version': '0.3.3',
    'description': 'Python wrapper for interacting with the Metabase API.',
    'long_description': '# metabase-python\n[![main](https://github.com/chasleslr/metabase-python/actions/workflows/main.yml/badge.svg)](https://github.com/chasleslr/metabase-python/actions/workflows/main.yml)\n[![codecov](https://codecov.io/gh/chasleslr/metabase-python/branch/main/graph/badge.svg?token=15G7HOQ1CM)](https://codecov.io/gh/chasleslr/metabase-python)\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n\nAn unofficial Python library for the [Metabase API](https://www.metabase.com/learn/administration/metabase-api).\n\n\n## Installation\n\n```\npip install metabase-python\n```\n\n## Usage\n\n### Connection\n\nStart by creating an instance of Metabase with your credentials.\n```python\nfrom metabase import Metabase\n\nmetabase = Metabase(\n    host="<host>",\n    user="<username/email>",\n    password="<password>",\n)\n```\n\n### Interacting with Endpoints\nYou can then interact with any of the supported endpoints through the classes included in this package. Methods that\ninstantiate an object from the Metabase API require the `using` parameter which expects an instance of `Metabase` such\nas the one we just instantiated above. All changes are reflected in Metabase instantly.\n\n```python\nfrom metabase import User\n\n# get all objects\nusers = User.list(using=metabase)\n\n# get an object by ID\nuser = User.get(1, using=metabase)\n\n# attributes are automatically loaded and available in the instance\nif user.is_active:\n    print("User is active!")\n\n# update any available attribute\nuser.update(is_superuser=True)\n\n# delete an object\nuser.delete()\n\n# create an object\nnew_user = User.create(\n    using=metabase,\n    first_name="<first_name>",\n    last_name="<last_name>",\n    email="<email>",\n    password="<password>"\n)\n```\n\nThe methods `.list()`, `.get()`, `.create()`, `.update()`, `.delete()` are available on all\nendpoints that support them in Metabase API.\n\nSome endpoints also support additional methods:\n\n```python\nfrom metabase import User\n\nuser = User.get(1, using=metabase)\n\nuser.reactivate()   # Reactivate user\nuser.send_invite()  # Resend the user invite email for a given user.\n```\n\nHere\'s a slightly more advanced example:\n```python\nfrom metabase import User, PermissionGroup, PermissionMembership\n\n# create a new PermissionGroup\nmy_group = PermissionGroup.create(name="My Group", using=metabase)\n\nfor user in User.list():\n    # add all users to my_group\n    PermissionMembership.create(\n        group_id=my_group.id,\n        user_id=user.id,\n        using=metabase,\n    )\n```\n\n### Querying & MBQL\n\nYou can also execute queries and get results back as a Pandas DataFrame. You can provide the exact MBQL, or use\nthe `Query` object to compile MBQL (i.e. Metabase Query Language) from Python classes included in this package.\n\n```python\nfrom metabase import Dataset, Query, Count, GroupBy, TemporalOption\n\ndataset = Dataset.create(\n    database=1,\n    type="query",\n    query={\n        "source-table": 1,\n        "aggregation": [["count"]],\n        "breakout": ["field", 7, {"temporal-unit": "year"},],\n    },\n    using=metabase,\n)\n\n# compile the MBQL above using the Query object\ndataset = Dataset.create(\n    database=1,\n    type="query",\n    query=Query(\n        table_id=2,\n        aggregations=[Count()],\n        group_by=[GroupBy(id=7, option=TemporalOption.YEAR)]\n    ).compile(),\n    using=metabase\n)\n\ndf = dataset.to_pandas()\n```\n\nAs shown above, the `Query` object allows you to easily compile MBQL from Python objects. Here is a\nmore complete example:\n```python\nfrom metabase import Query, Sum, Average, Metric, Greater, GroupBy, BinOption, TemporalOption\n\nquery = Query(\n    table_id=5,\n    aggregations=[\n        Sum(id=5),                                  # Provide the ID for the Metabase field\n        Average(id=5, name="Average of Price"),     # Optionally, you can provide a name\n        Metric.get(5)                               # You can also provide your Metabase Metrics\n    ],\n    filters=[\n        Greater(id=1, value=5.5)                    # Filter for values of FieldID 1 greater than 5.5\n    ],\n    group_by=[\n        GroupBy(id=4),                              # Group by FieldID 4\n        GroupBy(id=5, option=BinOption.AUTO),       # You can use Metabase\'s binning feature for numeric fields\n        GroupBy(id=5, option=TemporalOption.YEAR)   # Or it\'s temporal option for date fields\n    ]\n)\n\nprint(query.compile())\n{\n    \'source-table\': 5,\n    \'aggregation\': [\n        [\'sum\', [\'field\', 5, None]],\n        [\'aggregation-options\', [\'avg\', [\'field\', 5, None]], {\'name\': \'Average of Price\', \'display-name\': \'Average of Price\'}],\n        ["metric", 5]\n    ],\n    \'breakout\': [\n        [\'field\', 4, None],\n        [\'field\', 5, {\'binning\': {\'strategy\': \'default\'}}],\n        [\'field\', 5, {\'temporal-unit\': \'year\'}]\n    ],\n    \'filter\': [\'>\', [\'field\', 1, None], 5.5]\n}\n```\n\nThis can also be used to more easily create `Metric` objects.\n\n```python\nfrom metabase import Metric, Query, Count, EndsWith, CaseOption\n\n\nmetric = Metric.create(\n    name="Gmail Users",\n    description="Number of users with a @gmail.com email address.",\n    table_id=2,\n    definition=Query(\n        table_id=1,\n        aggregations=[Count()],\n        filters=[EndsWith(id=4, value="@gmail.com", option=CaseOption.CASE_INSENSITIVE)]\n    ).compile(),\n    using=metabase\n)\n```\n\n\n\n## Endpoints\n\nFor a full list of endpoints and methods, see [Metabase API](https://www.metabase.com/docs/latest/api-documentation.html).\n\n| Endpoints             | Support    | Notes |\n|-----------------------|:----------:|-------|\n| Activity              |  ❌        |       |\n| Alert                 |  ❌        |       |\n| Automagic dashboards  |  ❌        |       |\n| Card                  |  ✅        |       |\n| Collection            |  ❌        |       |\n| Dashboard             |  ❌        |       |\n| Database              |  ✅        |       |\n| Dataset               |  ✅        |       |\n| Email                 |  ❌        |       |\n| Embed                 |  ❌        |       |\n| Field                 |  ✅        |       |\n| Geojson               |  ❌        |       |\n| Ldap                  |  ❌        |       |\n| Login history         |  ❌        |       |\n| Metric                |  ✅        |       |\n| Native query snippet  |  ❌        |       |\n| Notify                |  ❌        |       |\n| Permissions           |  ✅        |       |\n| Premium features      |  ❌        |       |\n| Preview embed         |  ❌        |       |\n| Public                |  ❌        |       |\n| Pulse                 |  ❌        |       |\n| Revision              |  ❌        |       |\n| Search                |  ❌        |       |\n| Segment               |  ✅        |       |\n| Session               |  ❌        |       |\n| Setting               |  ❌        |       |\n| Setup                 |  ❌        |       |\n| Slack                 |  ❌        |       |\n| Table                 |  ✅        |       |\n| Task                  |  ❌        |       |\n| Tiles                 |  ❌        |       |\n| Transform             |  ❌        |       |\n| User                  |  ✅        |       |\n| Util                  |  ❌        |       |\n\n## Contributing\nContributions are welcome!\n\n## License\nThis library is distributed under the MIT license.\n',
    'author': 'Charles Lariviere',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/chasleslr/metabase-python',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
